"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.useViewModel = useViewModel;

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _react = require("react");

var _context4 = require("./context");

var _resolveCore = require("resolve-core");

var _useClient = require("./use-client");

var _generic = require("./generic");

function useViewModel(modelName, aggregateIds, args, stateChangeCallback, eventReceivedCallback, queryOptions) {
  var context = (0, _react.useContext)(_context4.ResolveContext);
  (0, _context4.assertContext)(context);
  var client = (0, _useClient.useClient)();
  var viewModels = context.viewModels;
  var viewModel = viewModels.find(function (_ref) {
    var name = _ref.name;
    return name === modelName;
  });

  if (!viewModel) {
    throw Error("View model " + modelName + " not exist within context");
  }

  var actualArgs = (0, _generic.isSerializableMap)(args) ? args : undefined;
  var actualQueryOptions = (0, _resolveCore.firstOfType)(_generic.isOptions, stateChangeCallback, eventReceivedCallback, queryOptions);
  var actualStateChangeCallback = (0, _resolveCore.firstOfType)(_generic.isCallback, args, stateChangeCallback);
  var actualEventReceivedCallback = (0, _resolveCore.firstOfType)(_generic.isCallback, stateChangeCallback, eventReceivedCallback);
  actualEventReceivedCallback = actualEventReceivedCallback !== actualStateChangeCallback ? actualEventReceivedCallback : undefined;

  if (!actualStateChangeCallback) {
    throw Error("state change callback required");
  }

  var closure = (0, _react.useMemo)(function () {
    var initialState = viewModel.projection.Init ? viewModel.projection.Init() : null;
    return {
      initialState: initialState,
      cursor: null
    };
  }, []);
  var setState = (0, _react.useCallback)(function (state, initial) {
    closure.state = state;
    actualStateChangeCallback(closure.state, initial);
  }, []);
  var queryState = (0, _react.useCallback)( /*#__PURE__*/(0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee() {
    var result, data, _result$meta, url, cursor;

    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return client.query({
              name: modelName,
              aggregateIds: aggregateIds,
              args: actualArgs
            }, actualQueryOptions);

          case 2:
            result = _context.sent;

            if (result) {
              data = result.data, _result$meta = result.meta;
              _result$meta = _result$meta === void 0 ? {} : _result$meta;
              url = _result$meta.url, cursor = _result$meta.cursor;
              setState(data, false);
              closure.url = url;
              closure.cursor = cursor != null ? cursor : null;
            }

          case 4:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  })), []);
  var applyEvent = (0, _react.useCallback)(function (event) {
    if ((0, _generic.isCallback)(actualEventReceivedCallback)) {
      actualEventReceivedCallback(event);
    }

    setState(viewModel.projection[event.type](closure.state, event), false);
  }, []);
  var connect = (0, _react.useCallback)(function (done) {
    var asyncConnect = /*#__PURE__*/function () {
      var _ref3 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee2() {
        var _closure$url;

        var subscribe, subscription;
        return _regenerator["default"].wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return queryState();

              case 2:
                subscribe = client.subscribe((_closure$url = closure.url) != null ? _closure$url : '', closure.cursor, modelName, aggregateIds, function (event) {
                  return applyEvent(event);
                }, undefined, function () {
                  return queryState();
                });
                _context2.next = 5;
                return subscribe;

              case 5:
                subscription = _context2.sent;

                if (subscription) {
                  closure.subscription = subscription;
                }

                return _context2.abrupt("return", subscription);

              case 8:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      }));

      return function asyncConnect() {
        return _ref3.apply(this, arguments);
      };
    }();

    setState(closure.initialState, true);

    if (typeof done !== 'function') {
      return asyncConnect();
    }

    asyncConnect().then(function (result) {
      return done(null, result);
    })["catch"](function (error) {
      return done(error, null);
    });
    return undefined;
  }, []);
  var dispose = (0, _react.useCallback)(function (done) {
    var asyncDispose = /*#__PURE__*/function () {
      var _ref4 = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee3() {
        return _regenerator["default"].wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!closure.subscription) {
                  _context3.next = 3;
                  break;
                }

                _context3.next = 3;
                return client.unsubscribe(closure.subscription);

              case 3:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3);
      }));

      return function asyncDispose() {
        return _ref4.apply(this, arguments);
      };
    }();

    if (typeof done !== 'function') {
      return asyncDispose();
    }

    asyncDispose().then(function () {
      return done();
    })["catch"](function (error) {
      return done(error);
    });
    return undefined;
  }, []);
  return (0, _react.useMemo)(function () {
    return {
      connect: connect,
      dispose: dispose,
      initialState: closure.initialState
    };
  }, []);
}
//# sourceMappingURL=use-view-model.js.map